/** @file msxe370x.h
* @brief MSX-E370x utils functions
*
* @author S.Nahas
* @date Feb 3, 2009
*
* @version  $LastChangedRevision: 3021 $
* @version $LastChangedDate: 2008-06-11 10:16:10 +0200 (Wed, 11 Jun 2008) $
*
*
* @par LICENCE
* @verbatim
   Copyright (C) 2009  ADDI-DATA GmbH for the source code of this module.

        ADDI-DATA GmbH
        Airport boulevard B210
        D-77836 Rheinmuenster
        Tel: +49(0)7229/1847-0
        Fax: +49(0)7229/1847-200
        http://www.addi-data-com
        info@addi-data.com

   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
   Foundation; either version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.
   See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with
   this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

   You shoud also find the complete GPL in the COPYING file
   accompanying this source code.
   @endverbatim
*
*
*/




#ifndef MSXE_MSXE370X_H_
#define MSXE_MSXE370X_H_

#ifdef WIN32
	#include <stdint_compat.h>
	#define __INLINE__ __inline
#else
	#include <stdint.h>
	#define __INLINE__ inline
#endif

struct msxe370x_autorefresh_parameters /** Structure describing the auto-refresh acquisition */
{
	uint32_t TransducerSelection; 		/**< */
	uint32_t NbrOfChannel; 				/**< value computed from ChannelMask */
	uint32_t ChannelList[16]; 			/**< value computed from TransducerSelection */
	uint32_t ChannelMask;				/**< */
	uint32_t AverageMode;				/**< */
	uint32_t AverageValue;				/**< */
	uint32_t DivisionFactor;   			/**< */
	uint32_t TriggerAction;				/**< */
	uint16_t HardwareTriggerCount;		/**< */
	uint16_t HardwareTriggerFilterTime; /**< */
	uint32_t ByTriggerNbrOfSeqToAcquire;/**< */
	uint32_t DataFormat;				/**< */
	uint32_t Option2;					/**< */
	uint32_t Option3;					/**< */
	uint32_t Option4;					/**< */
};

//---------------------------------------------------------------------------------
/** interactively get parameters through command line for autorefresh mode
 *
 * @param params structure describing an autorefresh acquisition
 * @param NumberOfChannels Number of channels the module has
 *
 * @retval void (always succeeds if not otherwise interrupted)
 */
void msxe370x_ask_autorefresh_parameters(struct msxe370x_autorefresh_parameters* params, uint32_t NumberOfChannels);

/** request data format in autorefresh mode
 * may not be used if client use SOAP/MODBUS functions to fetch data
 * @param params structure describing an autorefresh acquisition
 */
void msxe370x_ask_autorefresh_format(struct msxe370x_autorefresh_parameters* params);

/** returns the number of field (unit of 32 bits)
 * the data server would send with the acquisition as described by "params"
 *
 * @param[in,out] params structure describing an auto-refresh acquisition
 *
 * @retval number of 32-bits units
 */
unsigned int msxe370x_autorefresh_compute_fields(struct msxe370x_autorefresh_parameters* params);

//---------------------------------------------------------------------------------

struct msxe370x_format
{
	const char *counter; /**< printf-like format for the counter, if any - receives the value as uint32_t */
	const char *dataheader; /**< printf-like format to be appended before the data - receives the number of the channel the data comes from as uint32_t */
	const char *hwtrigger; /**< printf-like format to use when printing the HW trigger information - receives the value as uint32_t - ignored in autorefresh mode */
	const char *digital; /**< printf-like format for acquisition in digital-format - receives the value as uint32_t */
	const char *analog; /**< printf-like format for acquisition in analog format - receives the value as 32-bits float */
	const char *datasep; /**< string to put between each data record */
	const char *recordterm; /**< string to put at the end of a data record */
};


int msxe370x_autorefresh_fprintf(FILE *stream, struct msxe370x_autorefresh_parameters *params, uint32_t data[], unsigned int count, struct msxe370x_format *format);
int msxe370x_autorefresh_printf(struct msxe370x_autorefresh_parameters *params, uint32_t data[], unsigned int count, struct msxe370x_format *format);
int msxe370x_getautorefreshvalues_fprintf(FILE* stream, struct msxe370x_autorefresh_parameters* params, uint32_t data[], unsigned int count, struct msxe370x_format *format);
int msxe370x_getautorefreshvalues_printf(struct msxe370x_autorefresh_parameters* params, uint32_t data[], unsigned int count, struct msxe370x_format *format);
//---------------------------------------------------------------------------------
struct msxe370x_sequence_parameters /** Structure describing the sequence acquisition */
{
	uint32_t TransducerSelection; 		/**< */
	uint32_t NbrOfChannel;				/**< */
	uint32_t ChannelList[16];			/**< */
	uint32_t DivisionFactor;			/**< */
	uint32_t NbrOfSequence;				/**< */
	uint32_t NbrMaxSequenceToTransfer;	/**< */
	uint32_t DelayMode;					/**< */
	uint32_t DelayTimeUnit;				/**< */
	uint32_t DelayValue;				/**< */
	uint32_t TriggerAction;				/**< */
	uint32_t HardwareTriggerCount;		/**< */
	uint32_t HardwareTriggerFilterTime;	/**< */
	uint32_t ByTriggerNbrOfSeqToAcquire;/**< */
	uint32_t DataFormat;				/**< */
	uint32_t Option2;					/**< */
	uint32_t Option3;					/**< */
	uint32_t Option4;					/**< */
};
//---------------------------------------------------------------------------------

/** interactively get parameters through command line for autorefresh mode
 *
 * @param Params : structure describing a sequence acquisition
 * @param NumberOfChannels : Number of channels the module has
 *
 * @retval void (always succeeds if not otherwise interrupted)
 */
void msxe370x_ask_sequence_parameters(struct msxe370x_sequence_parameters* Params, uint32_t NumberOfChannels);

/** request data format in sequence mode
 * may not be used if client use SOAP/MODBUS functions to fetch data
* @param params : structure describing a sequence acquisition
*/
 void msxe370x_ask_sequence_format(struct msxe370x_sequence_parameters* params);

/** returns the number of field (unit of 32 bits)
 * the data server would send with the acquisition as described by "params"
 *
 * @param params structure describing a sequence acquisition
 *
 * @retval number of 32-bits units
 */
unsigned int msxe370x_sequence_compute_fields(struct msxe370x_sequence_parameters* params);

int msxe370x_sequence_fprintf(FILE* stream, struct msxe370x_sequence_parameters* params, uint32_t data[], unsigned int count, struct msxe370x_format *format);
int msxe370x_sequence_printf(struct msxe370x_sequence_parameters* params, uint32_t data[], unsigned int count, struct msxe370x_format *format);

static __INLINE__ unsigned int msxe370x_autorefresh_format_has_timestamp(struct msxe370x_autorefresh_parameters* params)
{
	return (params->DataFormat & 0x1);
}

static __INLINE__ unsigned int msxe370x_autorefresh_format_is_analog(struct msxe370x_autorefresh_parameters* params)
{
	return ((params->DataFormat >> 1) & 0x1);
}

static __INLINE__ unsigned int msxe370x_autorefresh_format_is_inverted(struct msxe370x_autorefresh_parameters* params)
{
	return ((params->DataFormat >> 2) & 0x1);
}

static __INLINE__ unsigned int msxe370x_sequence_format_has_absolute_timestamp(struct msxe370x_sequence_parameters* params)
{
	return ((params->DataFormat) & 0x1);
}

static __INLINE__ unsigned int msxe370x_sequence_format_has_relative_timestamp(struct msxe370x_sequence_parameters* params)
{
	return ((params->DataFormat >> 4) & 0x1);
}

static __INLINE__ unsigned int msxe370x_sequence_format_has_sequence_counter(struct msxe370x_sequence_parameters* params)
{
	return ((params->DataFormat >> 1) & 0x1);
}

static __INLINE__ unsigned int msxe370x_sequence_format_is_analog(struct msxe370x_sequence_parameters* params)
{
	return ((params->DataFormat >> 2) & 0x1);
}

static __INLINE__ unsigned int msxe370x_sequence_format_is_inverted(struct msxe370x_sequence_parameters* params)
{
	return ((params->DataFormat >> 3) & 0x1);
}

static __INLINE__ unsigned int msxe370x_sequence_format_has_hwtriggerinputinformation(struct msxe370x_sequence_parameters* params)
{
	return ((params->DataFormat >> 5) & 0x1);
}


#endif /* MSXE370X_H_ */
