using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections;


namespace ExtTemperatureRead
{
    public partial class MainForm : Form
    {
        /// <summary>
        /// Store the MSX-E object
        /// </summary>
        private Msxe371x msxe;

        /// <summary>
        /// Sensor version
        /// </summary>
        private bool tcVersion;

        /// <summary>
        /// Constructor of the GUI
        /// </summary>
        public MainForm()
        {
            tcVersion = false;
            InitializeComponent();
        }

        /// <summary>
        /// Event handler: occurs when the form is loaded for the first time
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            this.ipTextBox.Text = Properties.Settings.Default.ip;
            this.sensorSelectionComboBox.SelectedIndex = 0;
            this.gainSelectionCombobox.SelectedIndex = 0;
            this.frequencySelectionCombobox.SelectedIndex = 0;
            this.displayFormatCombobox.SelectedIndex = 0;
        }

        /// <summary>
        /// Convert date time 
        /// </summary>
        /// <param name="time_t"></param>
        /// <returns></returns>
        public static DateTime ConvertFromTime_T(long time_t)
        {
            DateTime dt70 = new DateTime(1970, 1, 1, 0, 0, 0, 0);
            long tickTotal = dt70.Ticks + (time_t * 10000000L);
            DateTime netTimeT = new DateTime(tickTotal);
            return netTimeT;
        }

        /// <summary>
        /// Event handler: occurs when the form is being closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            Properties.Settings.Default.ip = this.ipTextBox.Text;
            Properties.Settings.Default.Save();
        }

        /// <summary>
        /// Event handler: click on the "Initialise Connection" button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void initialiseConnectionButton_Click(object sender, EventArgs e)
        {
            // Check the validity of the IP address
            if (this.ipTextBox.Valid == false)
                return;

            // Update some graphics
            this.initialiseConnectionButton.Enabled = false;
            this.actionToolStripProgressBar.Visible = true;
            this.actionToolStripStatusLabel.Text = "Initialise the connection with the MSX-E system.";
            this.actionToolStripStatusLabel.Visible = true;

            // Start the connection background worker
            this.connectionBackgroundWorker.RunWorkerAsync(this.ipTextBox.Text);
        }

        /// <summary>
        /// Job of the connection background worker
        /// The connection task is done in background since it may takes some time to initialise the system
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void connectionBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            int retval = 0;
            string error = string.Empty;

            // Construct the new MSX-E object
            Msxe371x m = new Msxe371x(this.ipTextBox.Text);

            // Do the connection
            retval = m.Connect(out error);
            e.Result = new ConnectionWorkerResult(m, retval, error);
        }

        /// <summary>
        /// Event handler: Connection background worker is completed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void connectionBackgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ConnectionWorkerResult r = (ConnectionWorkerResult)e.Result;

            // Error ?
            if (r.retval != 0)
            {
                string error = string.Empty;
                this.actionToolStripProgressBar.Visible = false;
                this.actionToolStripStatusLabel.Text = "Error occurred.";
                // Reset the system type
                this.systemInformationTextBox.Text = string.Empty;
                // Display the right text
                switch (r.retval)
                {
                    case -1:
                        error = "The selected IP address is wrong. Please change it and retry.";
                        break;
                    case -2:
                        error = "A system error occured: " + r.error;
                        break;
                    case -3:
                        error = "The selected system cannot be used with this sample.";
                        break;
                    case -4:
                        error = "It is not possible to open a connection with the selected system.";
                        break;
                    default:
                        error = "An error occured: " + r.error;
                        break;
                }
                // Display the error
                MessageBox.Show(error, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                // Store the object
                this.msxe = r.msxe;

                // Deactivate and activate some button 
                this.actionToolStripProgressBar.Visible = false;
                this.actionToolStripStatusLabel.Text = "Connection opened with the system: " + r.msxe.IPAddress;
                
                // Display the system type
                this.systemInformationTextBox.Text = r.msxe.SystemType;

                this.externalTemperatureGroupBox.Enabled = true;

                // Clear items before adding new
                sensorSelectionComboBox.Items.Clear();
                gainSelectionCombobox.Items.Clear();
                displayFormatCombobox.Items.Clear();
                frequencySelectionCombobox.Items.Clear();

                gainSelectionCombobox.Items.Add("0 (Automode)");
                gainSelectionCombobox.SelectedIndex = 0;
                
                if (this.msxe.SystemType.Contains("{2000}"))
                {
                    // Add sensor type to list
                    sensorSelectionComboBox.Items.Add("TC type B");
                    sensorSelectionComboBox.Items.Add("TC type E");
                    sensorSelectionComboBox.Items.Add("TC type J");
                    sensorSelectionComboBox.Items.Add("TC type K");
                    sensorSelectionComboBox.Items.Add("TC type N");
                    sensorSelectionComboBox.Items.Add("TC type R");
                    sensorSelectionComboBox.Items.Add("TC type S");
                    sensorSelectionComboBox.Items.Add("TC type T");
                    tcVersion = true;
                }
                else
                {
                    // Add sensor type to list
                    sensorSelectionComboBox.Items.Add("PT100");
                    sensorSelectionComboBox.Items.Add("PT200");
                    sensorSelectionComboBox.Items.Add("PT500");
                    sensorSelectionComboBox.Items.Add("PT1000");

                    // Add mOhm
                    displayFormatCombobox.Items.Add("mOhm");

                    // Add sensor type to list
                    frequencySelectionCombobox.Items.Add("10");
                }

                // Add frequency to list
                frequencySelectionCombobox.Items.Add("50");
                frequencySelectionCombobox.Items.Add("240");
                frequencySelectionCombobox.SelectedIndex = 0;
                displayFormatCombobox.Items.Add("C");
                displayFormatCombobox.Items.Add("F");
                displayFormatCombobox.SelectedIndex = 0;
            }

            // Enable the "Initialise Connection" button
            this.initialiseConnectionButton.Enabled = true;
            this.sensorSelectionComboBox.SelectedIndex = 0;
        }

        /// <summary>
        /// Event handler: click on the "Initialise temperature sensor" button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void inititalizeTemperatureSensorButton_Click(object sender, EventArgs e)
        {       
            // Default
            string error = string.Empty;
            int retval = 0;
            ulong connectedSensorType = (ulong)sensorSelectionComboBox.SelectedIndex;
            ulong convertMode = (tcVersion) ? (ulong)displayFormatCombobox.SelectedIndex + 1 : (ulong)displayFormatCombobox.SelectedIndex;
            ulong frequencySelection = (tcVersion) ? (ulong)frequencySelectionCombobox.SelectedIndex + 1 : (ulong)frequencySelectionCombobox.SelectedIndex;
            ulong gainSelection = 0;

            // Compute correct sensor type
            if (!tcVersion)
            {
                switch (sensorSelectionComboBox.SelectedIndex)
                {
                    case 0:
                        connectedSensorType = 100;
                        break;
                    case 1:
                        connectedSensorType = 200;
                        break;
                    case 2:
                        connectedSensorType = 500;
                        break;
                    case 3:
                        connectedSensorType = 1000;
                        break;
                    default:
                        return;
                }
            }

            // Compute the frequency
            switch (frequencySelection)
            {
                case 0:
                    frequencySelection = 10;
                    break;

                case 1:
                    frequencySelection = 50;
                    break;

                case 2:
                    frequencySelection = 240;
                    break;
                default:
                    return;
            }

            // Compute the gain
            switch (gainSelectionCombobox.SelectedIndex)
            {
                case 0:
                    gainSelection = 0;
                    break;
                case 1:
                    gainSelection = 1;
                    break;
                default:
                    gainSelection = 0;
                    return;
            }

            // Display the unit
            this.unitlabel.Text = this.displayFormatCombobox.SelectedItem.ToString();
            
            // Initialisation 
            retval = this.msxe.ExternalTemperatureInit(connectedSensorType,
                convertMode,
                gainSelection,
                frequencySelection,
                0, //0 
                out error);

            if (retval != 0)
            {
                switch (retval)
                {
                    case -1:
                        error = "The selected IP address is wrong. Please change it and retry.";
                        break;
                    case -2:
                        error = "A system error occured: " + error;
                        break;
                    case -3:
                        error = "The selected system cannot be used with this sample.";
                        break;
                    case -4:
                        error = "It is not possible to open a connection with the selected system.";
                        break;
                    default:
                        error = "An error occured: " + error;
                        break;
                }

                // Display the error
                MessageBox.Show(error, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            // Update some graphics
            this.inititaliseTemperatureSensorButton.Enabled = false;
            this.temperatureBackgroundWorker.RunWorkerAsync(this.msxe);
            this.temperatureReleaseButton.Enabled = true;
            this.sensorSelectionComboBox.Enabled = false;
            this.gainSelectionCombobox.Enabled = false;
            this.frequencySelectionCombobox.Enabled = false;
            this.displayFormatCombobox.Enabled = false;
        }
        
        /// <summary>
        /// Release the temperature
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void temperatureReleaseButton_Click(object sender, EventArgs e)
        {
            //
            this.temperatureBackgroundWorker.CancelAsync();
            this.inititaliseTemperatureSensorButton.Enabled = true;
            this.sensorSelectionComboBox.Enabled = true;
            this.gainSelectionCombobox.Enabled = true;
            this.frequencySelectionCombobox.Enabled = true;
            this.displayFormatCombobox.Enabled = true;
            this.temperatureReleaseButton.Enabled = false;

            // Default
            string error = string.Empty;
            int retval = 0;

            try
            {
                // Release external temperature
                retval = this.msxe.ExternalTemperatureRelease(out error);

                if (retval != 0)
                {

                    switch (retval)
                    {
                        case -1:
                            error = "means an system error occurre.";
                            break;
                        default:
                            error = "An error occured: " + error;
                            break;
                    }

                    MessageBox.Show("ExternalTemperatureRelease Error : " + retval.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("ExternalTemperatureRelease Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
        }
        
        /// <summary>
        /// Main job Backgroundworker read te temperature value 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TemperatureBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            int retval = 0;
            string error = string.Empty;
            BackgroundWorker worker = (BackgroundWorker)sender;
            Msxe371x sys = (Msxe371x)e.Argument;

            for (; ;)
            {
                // Value for the acquisition
                ulong value = 0;
                ulong ts = 0;
                ulong tus = 0;

                // Cancel ?
                if (worker.CancellationPending)
                    return;

                // Read the temperature
                retval = sys.ExternalTemperatureRead(out value, out tus, out ts, out error);
                if (retval != 0)
                {
                    switch (retval)
                    {
                        case -1:
                            error = "The selected IP address is wrong. Please change it and retry.";
                            break;
                        case -2:
                            error = "A system error occured: " + error;
                            break;
                        case -3:
                            error = "The selected system cannot be used with this sample.";
                            break;
                        case -4:
                            error = "It is not possible to open a connection with the selected system.";
                            break;
                        default:
                            error = "An error occured: " + error;
                            break;
                    }

                    // Display the error
                    MessageBox.Show(error, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                // Report some progress
                worker.ReportProgress(1, new ReadWorkerValues(ts, tus, value));

                System.Threading.Thread.Sleep(20);
            }
        }

        /// <summary>
        /// Event handler : read background worker report some progress
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void temperatureBackgroundWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            ReadWorkerValues v = (ReadWorkerValues)e.UserState;

            // Float ?
            if (tcVersion || (!tcVersion && (displayFormatCombobox.SelectedIndex != 0)))
            {
                byte[] array = BitConverter.GetBytes((uint)v.value);
                this.temperatureLabel.Text = BitConverter.ToSingle(array, 0).ToString();
            }
            else
            {
                this.temperatureLabel.Text = v.value.ToString() + " mOhm";
            }

            this.secondlabel.Text = v.ts.ToString();
            this.milisecondlabel.Text = v.tus.ToString();
        }
    }
}